#include <nori/integrators/path.h>
#include <nori/shapes/shape.h>
#include <nori/bsdfs/bsdf.h>
#include <nori/core/scene.h>
#include <nori/warp/warp.h>
#include <nori/samplers/sampler.h>
#include <nori/emitters/emitter.h>

NORI_NAMESPACE_BEGIN

PathIntegrator::PathIntegrator(const PropertyList &props)
	: m_terminationParam(props.getFloat("termination-param", 2)) 
	, m_nSamples(props.getInteger("nSamples", 1)){

	std::string termination = props.getString("termination");
	if (termination == "russian-roulette") {
		m_termination = Termination::ERussianRoulette;
	}else {
		m_termination = Termination::EMaxDepth;
	}

	m_directMeasure = getMeasure(props.getString("direct-measure"));
	m_indirectMeasure = getMeasure(props.getString("indirect-measure"));
	m_directWarpType = Warp::getWarpType(m_directMeasure, props.getString("direct-warp"));
	m_indirectWarpType = Warp::getWarpType(m_indirectMeasure, props.getString("indirect-warp")); 

	if (props.getBoolean("isExplicit", false))
		m_Li = &PathIntegrator::explicitLi;
	else
		m_Li = &PathIntegrator::implicitLi;
}

Color3f PathIntegrator::implicitLi(const Scene* scene, Sampler* sampler, const Ray3f &ray) const {
	// ECSE689: Implement implicit path tracing
	return Color3f(0.f);
}

Color3f PathIntegrator::explicitLi(const Scene* scene, Sampler* sampler, const Ray3f &ray) const {
	// ECSE689: Implement explicit path tracing
	return Color3f(0.f);
}

Color3f PathIntegrator::Li(const Scene *scene, Sampler *sampler, const Ray3f &ray) const {
	return m_Li(this, scene, sampler, ray);
}

bool PathIntegrator::stopPath(uint32_t currentDepth) const {
	// ECSE689: Implement max depth and russian roulette termination here
	bool stop = false;

	switch (m_termination)
	{
	case Termination::EMaxDepth: {
		uint32_t maxDepth = static_cast<uint32_t>(m_terminationParam);
		// ...
	}
	break;
	case Termination::ERussianRoulette: {
		// ...
	}
	break;
	}

	return stop;
}

std::string PathIntegrator::toString() const {
	return tfm::format(
		"PathIntegrator[]"
	);
}

NORI_REGISTER_CLASS(PathIntegrator, "path");
NORI_NAMESPACE_END